@@ -243,6 +243,11 @@ pub fn sort_to_indices(
         DataType::Interval(IntervalUnit::DayTime) => {
             sort_primitive::<IntervalDayTimeType, _>(values, v, n, cmp, &options, limit)
         }
+        DataType::Interval(IntervalUnit::MonthDayNano) => {
+            sort_primitive::<IntervalMonthDayNanoType, _>(
+                values, v, n, cmp, &options, limit,
+            )
+        }
         DataType::Duration(TimeUnit::Second) => {
             sort_primitive::<DurationSecondType, _>(values, v, n, cmp, &options, limit)
         }
@@ -471,7 +476,8 @@ fn sort_boolean(
     let mut result = MutableBuffer::new(result_capacity);
     // sets len to capacity so we can access the whole buffer as a typed slice
     result.resize(result_capacity, 0);
-    let result_slice: &mut [u32] = result.typed_data_mut();
+    // Safety: the buffer is always treated as `u32` in the code below
+    let result_slice: &mut [u32] = unsafe { result.typed_data_mut() };
 
     if options.nulls_first {
         let size = nulls_len.min(len);
@@ -559,7 +565,8 @@ where
     let mut result = MutableBuffer::new(result_capacity);
     // sets len to capacity so we can access the whole buffer as a typed slice
     result.resize(result_capacity, 0);
-    let result_slice: &mut [u32] = result.typed_data_mut();
+    // Safety: the buffer is always treated as `u32` in the code below
+    let result_slice: &mut [u32] = unsafe { result.typed_data_mut() };
 
     if options.nulls_first {
         let size = nulls_len.min(len);
